// Get the backend component
var javaComponent = Components.classes["@jsug.tuwien.at/je-backend;1"]
      .getService(Components.interfaces.nsISupports);
      
var dicTreeView = { 
  
  visibleData : [],  
   
  treeBox: null,  
  selection: null,  
   
  get rowCount()                     { return this.visibleData.length; },  
  setTree: function(treeBox)         { this.treeBox = treeBox; },  
  getCellText: function(idx, column) {  
      return this.visibleData[idx][0]; 
  },  
  isContainer: function(idx)         { return this.visibleData[idx][1]; },  
  isContainerOpen: function(idx)     { return this.visibleData[idx][3]; },
  isContainerInContainer: function(idx) { return this.visibleData[idx][2]; },   
  isSeparator: function(idx)         { return false; },  
  isSorted: function()               { return false; },  
  isEditable: function(idx, column)  { return false; }, 
  
  /**
   * Needs a better implementation ?!
   */     
  isContainerEmpty: function(idx) { 
    if (this.getLevel(idx) == 0)
      return false;
    if (this.isContainerOpen(idx) && (this.getLevel(idx) < (this.getLevel(idx + 1))))
      return false;
    if (!this.isContainerOpen(idx))
      return false;
    return true;
  },  
   
  getParentIndex: function(idx) {  
    var l = this.getLevel(idx);
    if (l == 0) return -1;  
    for (var t = idx - 1; t >= 0 ; t--) {  
      if (this.getLevel(t) < l) return t;  
    }  
  },  
  getLevel: function(idx) { 
    if (this.isContainerInContainer(idx)) return 1;  
    if (this.isContainer(idx)) return 0;
    return 2;  
  },  
  hasNextSibling: function(idx, after) {  
    var thisLevel = this.getLevel(idx);  
    for (var t = after + 1; t < this.visibleData.length; t++) {  
      var nextLevel = this.getLevel(t);  
      if (nextLevel == thisLevel) return true;  
      if (nextLevel < thisLevel) break;  
    }  
    return false;  
  },  
  toggleOpenState: function(idx) {  
    var item = this.visibleData[idx];  
    // Check if it is a container.
    if (!item[1]) return;  
  
    // Check if it is an opened container.  
    if (item[3]) {  
      item[3] = false;  
  
      var thisLevel = this.getLevel(idx);  
      var deletecount = 0;  
      for (var t = idx + 1; t < this.visibleData.length; t++) {  
        if (this.getLevel(t) > thisLevel) deletecount++;  
        else break;  
      }  
      if (deletecount) {  
        this.visibleData.splice(idx + 1, deletecount);  
        this.treeBox.rowCountChanged(idx + 1, -deletecount);  
      }  
    }  
    else { 
      item[3] = true; 
      // Check if it is a normal container (the top level one).
      if (!item[2]) { 
        this.visibleData.splice(idx + 1, 0, ["phrases", true, true, false]);
        this.visibleData.splice(idx + 2, 0, ["regexps", true, true, false]);   
        this.treeBox.rowCountChanged(idx + 1, 2);
      } else {
        var label = this.visibleData[this.getParentIndex(idx)][0];
        // Check if phrases or regexps need to be inserted.
        if (this.visibleData[idx][0] == "phrases")
          var toinsert = javaComponent.wrappedJSObject.getPhrases(label);
        else
          var toinsert = javaComponent.wrappedJSObject.getRegexps(label);
          
        for (var i = 0; i < toinsert.length; i++) { 
          this.visibleData.splice(idx + i + 1, 0, [toinsert[i], false]);  
        }  
        this.treeBox.rowCountChanged(idx + 1, toinsert.length);
      } 
    }  
    this.treeBox.invalidateRow(idx);  
  },  
   
  getImageSrc: function(idx, column) {},  
  getProgressMode : function(idx,column) {},  
  getCellValue: function(idx, column) {},  
  cycleHeader: function(col, elem) {},  
  selectionChanged: function() {},  
  cycleCell: function(idx, column) {},  
  performAction: function(action) {},  
  performActionOnCell: function(action, index, column) {},  
  getRowProperties: function(idx, column, prop) {},  
  getCellProperties: function(idx, column, prop) {},  
  getColumnProperties: function(column, element, prop) {},  
};  
   
function dictInit() { 
  // Add eventlisteners to the ws-tree
  dicTree = document.getElementById("dict-tree");
  //dicTree.addEventListener("focus", treeSelected, true);
  dicTree.addEventListener("select", dicTreeSelected, true);
  dicTree.addEventListener("blur", dicTreeUnselected, true);
  refreshDicTree(); 
}

/**
 * Refreshes the dictionary tree.
 */ 
function refreshDicTree() {
  dicTreeView.visibleData = [];
  // Fill webservice tree with categories
  dictEntities = javaComponent.wrappedJSObject.getEntities();
  for (var i = 0; i < dictEntities.length; i++) {
    dicTreeView.visibleData[i] = [dictEntities[i], true, false, false];
  }
  //dicTree.boxObject.invalidate();
  dicTree.view = dicTreeView; 
}

/**
 * Activate edit & delete buttons on the dictionary tree when row selected.
 */ 
function dicTreeSelected(event) {
  if (event.target.id == "dict-tree") {
    if (dicTree.view.selection.getRangeCount() > 0) {
      document.getElementById("dictEditBtn").setAttribute('disabled','false');
      document.getElementById("dictDeleteBtn").setAttribute('disabled','false');
    }
  }
}

/**
 * Deactivates edit & delete buttons on the dictionary tree when focus removed.
 */ 
function dicTreeUnselected(event) {
  if (event.target.id == "dict-tree") {
    if (dicTree.view.selection.getRangeCount() == 0) {
      document.getElementById("dictEditBtn").setAttribute('disabled','true');
      document.getElementById("dictDeleteBtn").setAttribute('disabled','true');
    }
  }
}

/**
 * Deletes specified prase/regexp for a specific category or the whole category 
 * from the dictionary. 
 */ 
function deleteDict() {
  var index = dicTree.currentIndex;
  var level = dicTree.view.getLevel(index);
  // If category is selected.
  if (level == 0) {
    var catName = dicTree.view.getCellText(index, dicTree.columns["dictNameCol"]);
    window.openDialog("chrome://je/content/dictDelConfirmdialog.xul", "confirm", "chrome, centerscreen", {category: catName, subcategory: "", type: "entity"});
  // If subcategory is selected (prases or regexps).
  } else if (level == 1) {
    var catIndex = dicTree.view.getParentIndex(index);
    var catName = dicTree.view.getCellText(catIndex, dicTree.columns["dictNameCol"]);
    var subCatName = dicTree.view.getCellText(index, dicTree.columns["dictNameCol"]);
    window.openDialog("chrome://je/content/dictDelConfirmdialog.xul", "confirm", "chrome, centerscreen", {category: catName, subcategory: subCatName, type: subCatName});
  // If leaf is selected.
  } else {
    var subCatIndex = dicTree.view.getParentIndex(index);
    var subCatName = dicTree.view.getCellText(subCatIndex, dicTree.columns["dictNameCol"]);
    var catIndex = dicTree.view.getParentIndex(subCatIndex);
    var catName = dicTree.view.getCellText(catIndex, dicTree.columns["dictNameCol"]);
    var selection = dicTree.view.getCellText(index, dicTree.columns["dictNameCol"]);
    if (subCatName == "phrases") {
      javaComponent.wrappedJSObject.deletePhrase(catName, selection);  
    } else {
      javaComponent.wrappedJSObject.deleteRegexp(catName, selection);
    } 
    refreshDicTree(); 
  }
}

/**
 * Deletes the whole category or its subcategory after confirmation.
 */ 
function confirmedDelete(category, subcategory) {
  if (subcategory == "") {
    javaComponent.wrappedJSObject.deleteEntity(category);
  } else if (subcategory == "phrases") {
    javaComponent.wrappedJSObject.deletePhrases(category, subcategory);
  } else {
    javaComponent.wrappedJSObject.deleteRegexps(category, subcategory);
  }
  refreshDicTree();
}