// Get the backend component
var javaComponent = Components.classes["@jsug.tuwien.at/je-backend;1"]
      .getService(Components.interfaces.nsISupports);
      
var wsTreeView = { 
  
  visibleData : [],  
   
  treeBox: null,  
  selection: null,  
   
  get rowCount()                     { return this.visibleData.length; },  
  setTree: function(treeBox)         { this.treeBox = treeBox; },  
  getCellText: function(idx, column) { 
    if (column.id == "urlCol") {
      return this.visibleData[idx][1];
    } else {
      return this.visibleData[idx][0]; 
    }    
  },  
  isContainer: function(idx)         { return this.visibleData[idx][2]; },  
  isContainerOpen: function(idx)     { return this.visibleData[idx][3]; },  
  isContainerEmpty: function(idx)    { return false; },  
  isSeparator: function(idx)         { return false; },  
  isSorted: function()               { return false; },  
  isEditable: function(idx, column)  { return false; },  
   
  getParentIndex: function(idx) {  
    if (this.isContainer(idx)) return -1;  
      for (var t = idx - 1; t >= 0 ; t--) {  
        if (this.isContainer(t)) return t;  
    }  
  },  
  getLevel: function(idx) {  
    if (this.isContainer(idx)) return 0;  
      return 1;  
  },  
  hasNextSibling: function(idx, after) {  
    var thisLevel = this.getLevel(idx);  
    for (var t = after + 1; t < this.visibleData.length; t++) {  
      var nextLevel = this.getLevel(t);  
      if (nextLevel == thisLevel) return true;  
      if (nextLevel < thisLevel) break;  
    }  
    return false;  
  },  
  toggleOpenState: function(idx) {  
    var item = this.visibleData[idx];  
    if (!item[2]) return;  
  
    if (item[3]) {  
      item[3] = false;  
  
      var thisLevel = this.getLevel(idx);  
      var deletecount = 0;  
      for (var t = idx + 1; t < this.visibleData.length; t++) {  
        if (this.getLevel(t) > thisLevel) deletecount++;  
        else break;  
      }  
      if (deletecount) {  
        this.visibleData.splice(idx + 1, deletecount);  
        this.treeBox.rowCountChanged(idx + 1, -deletecount);  
      }  
    }  
    else {  
      item[3] = true;  
   
      var label = this.visibleData[idx][0];  
      var toinsert = javaComponent.wrappedJSObject.getServices(label);  
      for (var i = 0; i < toinsert.length; i++) {
        var splitted = toinsert[i].split(";");  
        this.visibleData.splice(idx + i + 1, 0, [splitted[0], splitted[1], false]);  
      }  
      this.treeBox.rowCountChanged(idx + 1, toinsert.length);  
    }  
    this.treeBox.invalidateRow(idx);  
  },  
   
  getImageSrc: function(idx, column) {},  
  getProgressMode : function(idx,column) {},  
  getCellValue: function(idx, column) {},  
  cycleHeader: function(col, elem) {},  
  selectionChanged: function() {},  
  cycleCell: function(idx, column) {},  
  performAction: function(action) {},  
  performActionOnCell: function(action, index, column) {},  
  getRowProperties: function(idx, column, prop) {},  
  getCellProperties: function(idx, column, prop) {},  
  getColumnProperties: function(column, element, prop) {},  
};  
   
function wsInit() { 
  // Add eventlisteners to the ws-tree
  wsTree = document.getElementById("ws-tree");
  //wsTree.addEventListener("focus", treeSelected, true);
  wsTree.addEventListener("select", wsTreeSelected, true);
  wsTree.addEventListener("blur", wsTreeUnselected, true);
  refreshWsTree(); 
}

/**
 * Refreshes the webservice tree.
 */ 
function refreshWsTree() {
  wsTreeView.visibleData = [];
  // Fill webservice tree with categories
  wsCategories = javaComponent.wrappedJSObject.getCategories();
  for (var i = 0; i < wsCategories.length; i++) {
    wsTreeView.visibleData[i] = [wsCategories[i], "", true, false];
  }
  //wsTree.boxObject.invalidate();
  wsTree.view = wsTreeView; 
}

/**
 * Activate edit & delete buttons on a tree when row selected
 */ 
function wsTreeSelected(event) {
  if (event.target.id == "ws-tree") {
    if (wsTree.view.selection.getRangeCount() > 0) {
      document.getElementById("editBtn").setAttribute('disabled','false');
      document.getElementById("deleteBtn").setAttribute('disabled','false');
    }
  }
}

/**
 * Deactivates edit & delete buttons on a tree when focus removed
 */ 
function wsTreeUnselected(event) {
  if (event.target.id == "ws-tree") {
    if (wsTree.view.selection.getRangeCount() == 0) {
      document.getElementById("editBtn").setAttribute('disabled','true');
      document.getElementById("deleteBtn").setAttribute('disabled','true');
    }
  }
}

/**
 * Opens the dialog for adding or editing a service
 */ 
function openWsWindow(event) {
  // If add button clicked
  if (event.explicitOriginalTarget.id == "addBtn") {
    window.openDialog("chrome://je/content/ws.xul", "addwebservice", "chrome, centerscreen", {categories: wsCategories});
  // If edit button clicked
  } else {
    var index = wsTree.currentIndex;
    var name = wsTree.view.getCellText(index, wsTree.columns["nameCol"]); 
    var url = wsTree.view.getCellText(index, wsTree.columns["urlCol"]);
    if (url == "") {
      alert("Category can not be edited.");
    } else {
      var category = wsTree.view.getCellText(wsTree.view.getParentIndex(index), wsTree.columns["nameCol"]);
      window.openDialog("chrome://je/content/ws.xul", "editwebservice", "chrome, centerscreen", {categories: wsCategories, catName: category, wsName: name, wsUrl: url, edit: true});
    }
  }
}

/**
 * Opens the dialog for editing a service
 */ 
function openWsWindowForEdit() {
  var index = wsTree.currentIndex;
  var name = wsTree.view.getCellText(index, wsTree.columns["nameCol"]); 
  var url = wsTree.view.getCellText(index, wsTree.columns["urlCol"]);
  if (url == "") {
    alert("Category can not be edited.");
  } else {
    var category = wsTree.view.getCellText(wsTree.view.getParentIndex(index), wsTree.columns["nameCol"]);
    window.openDialog("chrome://je/content/editws.xul", "editwebservice", "chrome, centerscreen", {categories: wsCategories, catName: category, wsName: name, wsUrl: url});
  }
}

/**
 * Deletes specified webservice or the whole category
 */ 
function deleteWs() {
  var index = wsTree.currentIndex;
  var name = wsTree.view.getCellText(index, wsTree.columns["nameCol"]); 
  var url = wsTree.view.getCellText(index, wsTree.columns["urlCol"]);
  // Delete whole category
  if (url == "") {
    window.openDialog("chrome://je/content/wsDelConfirmDialog.xul", "confirm", "chrome, centerscreen", {category: name});
  // Delete selected webservice
  } else {
    var category = wsTree.view.getCellText(wsTree.view.getParentIndex(index), wsTree.columns["nameCol"]);
    javaComponent.wrappedJSObject.deleteService(category, name);
    alert("Service deleted.");
    refreshWsTree();    
  }
}

/**
 * Deletes category with all its webservices
 */ 
function deleteWsCategory(catName) {
  javaComponent.wrappedJSObject.deleteCategory(catName);
  alert("Category deleted.");
  refreshWsTree();  
} 