package backend;
import java.io.FileReader;
import java.util.Vector;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;


import backend.dictionary.DictChunker;
import backend.dictionary.DictManagement;
import backend.dictionary.xml.EntityMaster;
import backend.dictionary.xml.FoundEntity;
import backend.webservice.WSManagement;
import backend.webservice.xml.WSRootElement;
import backend.webservice.xml.WebService;



public class BackendHandler {

	private String dictfile = "dict.xml";
	private String wsfile = "webservice.xml";
	private JAXBContext dictContext = null;
	private JAXBContext wsContext = null;
	private EntityMaster entityMaster = null;
	private WSRootElement wsRoot = null;
	private Vector<FoundEntity> foundEntities;

	public static BackendHandler bHandler = null;
	
	public static BackendHandler getHandler() {
		if (bHandler == null) {
			bHandler = new BackendHandler();
		}
		return bHandler;
	}
	
	/**
	 * Initializes the backend.
	 * @param dictfile name of the dictionary file
	 */
	public void init(String dictfile, String wsfile) {
		this.dictfile = dictfile;
		this.wsfile = wsfile;
		
		try {
			//-- Unmarshalls the dictionary xml
			dictContext = JAXBContext.newInstance(EntityMaster.class);
			Unmarshaller um = dictContext.createUnmarshaller();
			entityMaster = (EntityMaster) um.unmarshal(new FileReader(dictfile));
			
			//-- Unmarshalls the webservices xml
			wsContext = JAXBContext.newInstance(WSRootElement.class);
			Unmarshaller um2 = wsContext.createUnmarshaller();
			wsRoot = (WSRootElement) um2.unmarshal(new FileReader(wsfile));
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		bHandler = this;
	}
	
	public String parse(String parseText) {
		return DictChunker.parse(parseText, bHandler);
	}
	
	public String[] getCategories() {
		Vector<String> categories = WSManagement.getCategories(bHandler);
		return categories.toArray(new String[categories.size()]);
	}
	
	//============
	// WEBSERVICES
	//============
	
	/**
	 * Returns names and URLs of services of specified category as Strings 
	 * in form: serviceName;serviceURL
	 * @param category Name of the category
	 * @return String[]
	 */
	public String[] getServices(String category) {
		Vector<WebService> servs = WSManagement.getServices(category, bHandler);
		String[] services = new String[servs.size()];
		for (int i = 0; i < servs.size(); i++) {
			WebService s = servs.elementAt(i);
			services[i] = s.getName() + ";" + s.getUrl();
		}
		return services;
	}
	
	public void addService(String category, String name, String url) {
		WSManagement.addService(category, name, url, bHandler);
	}
	
	public void deleteService(String category, String name) {
		WSManagement.deleteService(category, name, bHandler);
	}
	
	public void deleteCategory(String name){
		WSManagement.deleteCategory(name, bHandler);
	}
	
	//===========
	// DICTIONARY
	//===========
	
	public String[] getEntities() {
		Vector<String> entities = DictManagement.getEntities(bHandler);
		return entities.toArray(new String[entities.size()]);
	}
	
	public String[] getPhrases(String entityName) {
		Vector<String> phrases = DictManagement.getPhrases(entityName, bHandler);
		return phrases.toArray(new String[phrases.size()]);
	}
	
	public String[] getRegexps(String entityName) {
		Vector<String> regexps = DictManagement.getRegexps(entityName, bHandler);
		return regexps.toArray(new String[regexps.size()]);
	}
	
	public void deleteEntity(String entityName) {
		DictManagement.deleteEntity(entityName, bHandler);
	}
	
	public void deletePhrases(String entityName) {
		DictManagement.deletePhrases(entityName, bHandler);
	}
	
	public void deleteRegexps(String entityName) {
		DictManagement.deleteRegexps(entityName, bHandler);
	}
	
	public void deletePhrase(String entityName, String phrase) {
		DictManagement.deletePhrase(entityName, phrase, bHandler);
	}
	
	public void deleteRegexp(String entityName, String regexp) {
		DictManagement.deleteRegexp(entityName, regexp, bHandler);
	}
	
	//==================
	// GETTERS & SETTERS
	//==================
	
	public String getDictfile() {
		return dictfile;
	}
	
	public String getWsfile() {
		return wsfile;
	}

	public JAXBContext getDictContext() {
		return dictContext;
	}
	
	public JAXBContext getWsContext() {
		return wsContext;
	}

	public EntityMaster getEntityMaster() {
		return entityMaster;
	}
	
	public WSRootElement getWsRootElement() {
		return wsRoot;
	}

	public Vector<FoundEntity> getFoundEntities() {
		return foundEntities;
	}

	public void setEntities(Vector<FoundEntity> foundEntities) {
		this.foundEntities = foundEntities;
	}	
}
