package backend.dictionary;

import java.util.Collections;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import backend.BackendHandler;
import backend.dictionary.xml.EntityType;
import backend.dictionary.xml.FoundEntity;

import com.aliasi.chunk.Chunk;
import com.aliasi.chunk.Chunker;
import com.aliasi.chunk.Chunking;
import com.aliasi.chunk.RegExChunker;
import com.aliasi.dict.DictionaryEntry;
import com.aliasi.dict.ExactDictionaryChunker;
import com.aliasi.dict.MapDictionary;
import com.aliasi.tokenizer.IndoEuropeanTokenizerFactory;

public class DictChunker {
	
	/**
	 * 
	 * @return the new (parsed) text
	 */
	public static String parse(String parseText, BackendHandler bHandler) {
		//String newText = parseText;
				
		String text = parseText;
		bHandler.setEntities(new Vector<FoundEntity>());
		MapDictionary<String> dict = new MapDictionary<String>();

		Iterator<EntityType> ent = bHandler.getEntityMaster().getEntityTypes().iterator();

		while (ent.hasNext()) {
			EntityType e = ent.next();
			Iterator<String> entries = e.getPhrase().iterator();
			if(e.getRegex()!=null){
				Iterator<String> regs = e.getRegex().iterator();
				while(regs.hasNext()){
				regexChunk(regs.next(),e.getName(), text, bHandler);
				}
			}
			while (entries.hasNext()) {
				String value = entries.next();
				dict.addEntry(new DictionaryEntry<String>(value, e.getName()));
			}

		}

		ExactDictionaryChunker finder = new ExactDictionaryChunker(dict,
				IndoEuropeanTokenizerFactory.FACTORY, true, true);
		chunk(finder, text, BackendHandler.getHandler());

		String newText = "";
		
		// Make the text lowercase to check tagging
		String lowerText = text.toLowerCase();
		int pointer = 0;
		int idCount = 0;
		
		// Sort entities ascending the start
		Collections.sort(bHandler.getFoundEntities());
		
		//System.out.println(text);
		for (FoundEntity fe : bHandler.getFoundEntities()) {
			
			// Skip everything inside an HTML tag
			if ((lowerText.lastIndexOf("<", fe.getStart()) > lowerText.lastIndexOf(">", fe.getStart()))) {
				continue;
			}
			
			// Skip everything inside an SCRIPT tag
			if ((lowerText.lastIndexOf("<script", fe.getStart()) > lowerText.lastIndexOf("/script>", fe.getStart()))) {
				continue;
			}
			EntityType entity = DictManagement.getEntity(fe.getType(), bHandler);			
			
			// Create replacement
			String repl = "<span name='markedSpan' style='background-color: " + entity.getBgcolor() + "; color: " + entity.getColor() + ";' " 
				+ "id='ganesha_" + idCount + "' " 
				+ "class='ganesha_" + fe.getType() + "'" + ">"
				+ fe.getPhrase() + "</span>";
			newText += text.substring(pointer, fe.getStart()) + repl;
			pointer = fe.getEnd();
			idCount++;		
		}
		
		newText += text.substring(pointer);
		//System.out.println(newText);
		
		return newText;
	}
	
	//========
	// PRIVATE
	//========
	
	private static void chunk(ExactDictionaryChunker chunker, String text, BackendHandler bHandler) {

		Chunking chunking = chunker.chunk(text);
		for (Chunk chunk : chunking.chunkSet()) {
			int start = chunk.start();
			int end = chunk.end();
			String type = chunk.type();
			String phrase = text.substring(start, end);
			FoundEntity fent = new FoundEntity(phrase, start, end, type);
			bHandler.getFoundEntities().add(fent);
		}

	}

	private static void regexChunk(String regex, String entityType, String text, BackendHandler bHandler) {

		Chunker chunker = new RegExChunker(regex, entityType, 0.0);
		Chunking chunking = chunker.chunk(text);
		Set<Chunk> chunkSet = chunking.chunkSet();
		Iterator<Chunk> it = chunkSet.iterator();
		while (it.hasNext()) {
			Chunk chunk = (Chunk) it.next();
			int start = chunk.start();
			int end = chunk.end();
			String type = chunk.type();
			String phrase = text.substring(start, end);
			FoundEntity fent = new FoundEntity(phrase, start, end, type);
			bHandler.getFoundEntities().add(fent);
		}
	}
}
